@@ -6,7 +6,7 @@ echo aclocal...
     exit 1
 }
 
-aclocal -I ./scripts -I . ${ACLOCAL_FLAGS}
+aclocal -I ./scripts -I . ${ACLOCAL_FLAGS} || exit 1
 
 echo autoheader...
 (autoheader --version) < /dev/null > /dev/null 2>&1 || {
@@ -14,16 +14,16 @@ echo autoheader...
     exit 1
 }
 
-autoheader
+autoheader || exit 1
 
 echo -n "libtoolize... "
 if ( (glibtoolize --version) < /dev/null > /dev/null 2>&1 ); then
     echo "using glibtoolize"
-    glibtoolize --automake --copy --force
+    glibtoolize --automake --copy --force || exit 1
 
 elif ( (libtoolize --version) < /dev/null > /dev/null 2>&1 ) ; then
     echo "using libtoolize"
-    libtoolize --automake --copy --force
+    libtoolize --automake --copy --force || exit 1
 
 else
     echo "libtoolize nor glibtoolize not found"
@@ -36,7 +36,7 @@ echo automake...
     exit 1
 }
 
-automake --add-missing --copy --gnu
+automake --add-missing --copy --gnu || exit 1
 
 echo autoconf...
 (autoconf --version) < /dev/null > /dev/null 2>&1 || {
@@ -44,7 +44,7 @@ echo autoconf...
     exit 1
 }
 
-autoconf
+autoconf || exit 1
 
 echo ready to configure
 
