@@ -39,8 +39,7 @@
 
 #include <string>
 #include <vector>
-#include <rak/error_number.h>
-#include <rak/functional.h>
+#include <tr1/functional>
 
 #include "chunk.h"
 #include "chunk_handle.h"
@@ -50,6 +49,7 @@ namespace torrent {
 
 class ChunkManager;
 class Content;
+class download_data;
 class DownloadWrapper;
 class FileList;
 
@@ -59,9 +59,9 @@ public:
   typedef std::vector<ChunkListNode>          base_type;
   typedef std::vector<ChunkListNode*>         Queue;
 
-  typedef rak::mem_fun2<FileList, Chunk*, uint32_t, int>           SlotCreateChunk;
-  typedef rak::const_mem_fun0<FileList, uint64_t>                  SlotFreeDiskspace;
-  typedef rak::mem_fun1<DownloadWrapper, void, const std::string&> SlotStorageError;
+  typedef std::tr1::function<Chunk* (uint32_t, int)>    slot_chunk_index;
+  typedef std::tr1::function<uint64_t ()>               slot_value;
+  typedef std::tr1::function<void (const std::string&)> slot_string;
 
   using base_type::value_type;
   using base_type::reference;
@@ -75,6 +75,7 @@ public:
   using base_type::end;
   using base_type::size;
   using base_type::empty;
+  using base_type::operator[];
 
   static const int sync_all          = (1 << 0);
   static const int sync_force        = (1 << 1);
@@ -84,32 +85,38 @@ public:
   static const int sync_ignore_error = (1 << 5);
 
   static const int get_writable      = (1 << 0);
-  static const int get_dont_log      = (1 << 1);
+  static const int get_blocking      = (1 << 1);
+  static const int get_dont_log      = (1 << 2);
+  static const int get_nonblock      = (1 << 3);
 
   static const int flag_active       = (1 << 0);
 
-  ChunkList() : m_manager(NULL), m_flags(0), m_chunk_size(0) {}
+  ChunkList() : m_data(NULL), m_manager(NULL), m_flags(0), m_chunk_size(0) {}
   ~ChunkList() { clear(); }
 
+  int                 flags() const                       { return m_flags; }
+
   void                set_flags(int flags)                { m_flags |= flags; }
   void                unset_flags(int flags)              { m_flags &= ~flags; }
   void                change_flags(int flags, bool state) { if (state) set_flags(flags); else unset_flags(flags); }
 
   uint32_t            chunk_size() const                  { return m_chunk_size; }
+  size_type           queue_size() const                  { return m_queue.size(); }
 
+  download_data*      data()                              { return m_data; }
+
+  void                set_data(download_data* data)       { m_data = data; }
   void                set_manager(ChunkManager* manager)  { m_manager = manager; }
   void                set_chunk_size(uint32_t cs)         { m_chunk_size = cs; }
 
   bool                has_chunk(size_type index, int prot) const;
 
-  void                resize(size_type s);
+  void                resize(size_type to_size);
   void                clear();
 
   ChunkHandle         get(size_type index, int flags = 0);
   void                release(ChunkHandle* handle, int flags = 0);
 
-  size_type           queue_size() const                      { return m_queue.size(); }
-
   // Replace use_timeout with something like performance related
   // keyword. Then use that flag to decide if we should skip
   // non-continious regions.
@@ -117,9 +124,9 @@ public:
   // Returns the number of failed syncs.
   uint32_t            sync_chunks(int flags);
 
-  void                slot_storage_error(SlotStorageError s)   { m_slotStorageError = s; }
-  void                slot_create_chunk(SlotCreateChunk s)     { m_slotCreateChunk = s; }
-  void                slot_free_diskspace(SlotFreeDiskspace s) { m_slotFreeDiskspace = s; }
+  slot_string&        slot_storage_error()  { return m_slot_storage_error; }
+  slot_chunk_index&   slot_create_chunk()   { return m_slot_create_chunk; }
+  slot_value&         slot_free_diskspace() { return m_slot_free_diskspace; }
 
   typedef std::pair<iterator, Chunk::iterator> chunk_address_result;
 
@@ -138,15 +145,16 @@ private:
 
   std::pair<int,bool> sync_options(ChunkListNode* node, int flags);
 
+  download_data*      m_data;
   ChunkManager*       m_manager;
   Queue               m_queue;
 
   int                 m_flags;
   uint32_t            m_chunk_size;
 
-  SlotStorageError    m_slotStorageError;
-  SlotCreateChunk     m_slotCreateChunk;
-  SlotFreeDiskspace   m_slotFreeDiskspace;
+  slot_string         m_slot_storage_error;
+  slot_chunk_index    m_slot_create_chunk;
+  slot_value          m_slot_free_diskspace;
 };
 
 }
