@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -41,6 +41,8 @@
 
 namespace torrent {
 
+class File;
+
 class ChunkPart {
 public:
   typedef enum {
@@ -49,11 +51,13 @@ public:
   } mapped_type;
 
   ChunkPart(mapped_type mapped, const MemoryChunk& c, uint32_t pos) :
-    m_mapped(mapped), m_chunk(c), m_position(pos) {}
+    m_mapped(mapped), m_chunk(c), m_position(pos), m_file(NULL), m_file_offset(0) {}
 
   bool                is_valid() const                      { return m_chunk.is_valid(); }
   bool                is_contained(uint32_t p) const        { return p >= m_position && p < m_position + size(); }
 
+  bool                has_address(void* p) const            { return (char*)p >= m_chunk.begin() && p < m_chunk.end(); }
+
   void                clear();
 
   mapped_type         mapped() const                        { return m_mapped; }
@@ -66,6 +70,11 @@ public:
 
   uint32_t            remaining_from(uint32_t pos) const    { return size() - (pos - m_position); }
 
+  File*               file() const                          { return m_file; }
+  uint64_t            file_offset() const                   { return m_file_offset; }
+
+  void                set_file(File* f, uint64_t f_offset)  { m_file = f; m_file_offset = f_offset; }
+
   bool                is_incore(uint32_t pos, uint32_t length = ~uint32_t());
   uint32_t            incore_length(uint32_t pos, uint32_t length = ~uint32_t());
 
@@ -74,6 +83,12 @@ private:
 
   MemoryChunk         m_chunk;
   uint32_t            m_position;
+
+  // Currently used to figure out what file and where a SIGBUS
+  // occurred. Can also be used in the future to indicate if a part is
+  // temporary storage, etc.
+  File*               m_file;
+  uint64_t            m_file_offset;
 };
 
 }
