@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -52,6 +52,10 @@
 
 namespace torrent {
 
+// Standard pair of up/down throttles.
+// First element is upload throttle, second element is download throttle.
+typedef std::pair<Throttle*, Throttle*> ThrottlePair;
+
 class LIBTORRENT_EXPORT ConnectionManager {
 public:
   typedef uint32_t                              size_type;
@@ -82,6 +86,7 @@ public:
 
   // Internal to libtorrent.
   static const uint32_t encryption_use_proxy        = (1 << 6);
+  static const uint32_t encryption_retrying         = (1 << 7);
 
   enum {
     handshake_incoming           = 1,
@@ -102,6 +107,8 @@ public:
   typedef sigc::slot4<void, const sockaddr*, int, int, const HashString*>   slot_handshake_type;
   typedef sigc::signal4<void, const sockaddr*, int, int, const HashString*> signal_handshake_type;
 
+  typedef sigc::slot1<ThrottlePair, const sockaddr*>  slot_address_throttle_type;
+
   ConnectionManager();
   ~ConnectionManager();
   
@@ -116,35 +123,26 @@ public:
   void                dec_socket_count()                      { m_size--; }
 
   size_type           size() const                            { return m_size; }
-
   size_type           max_size() const                        { return m_maxSize; }
-  void                set_max_size(size_type s)               { m_maxSize = s; }
 
   priority_type       priority() const                        { return m_priority; }
-  void                set_priority(priority_type p)           { m_priority = p; }
-
   uint32_t            send_buffer_size() const                { return m_sendBufferSize; }
-  void                set_send_buffer_size(uint32_t s);
-
   uint32_t            receive_buffer_size() const             { return m_receiveBufferSize; }
-  void                set_receive_buffer_size(uint32_t s);
-
   uint32_t            encryption_options()                    { return m_encryptionOptions; }
+
+  void                set_max_size(size_type s)               { m_maxSize = s; }
+  void                set_priority(priority_type p)           { m_priority = p; }
+  void                set_send_buffer_size(uint32_t s);
+  void                set_receive_buffer_size(uint32_t s);
   void                set_encryption_options(uint32_t options); 
 
-  // Propably going to have to make m_bindAddress a pointer to make it
-  // safe.
-  //
-  // Perhaps add length parameter.
-  //
-  // Setting the bind address makes a copy.
+  // Setting the addresses creates a copy of the address.
   const sockaddr*     bind_address() const                    { return m_bindAddress; }
-  void                set_bind_address(const sockaddr* sa);
-
   const sockaddr*     local_address() const                   { return m_localAddress; }
-  void                set_local_address(const sockaddr* sa);
-
   const sockaddr*     proxy_address() const                   { return m_proxyAddress; }
+
+  void                set_bind_address(const sockaddr* sa);
+  void                set_local_address(const sockaddr* sa);
   void                set_proxy_address(const sockaddr* sa);
 
   uint32_t            filter(const sockaddr* sa);
@@ -156,11 +154,12 @@ public:
   signal_handshake_type& signal_handshake_log()                          { return m_signalHandshakeLog; }
   sigc::connection       set_signal_handshake_log(slot_handshake_type s) { return m_signalHandshakeLog.connect(s); }
 
-  // The resolver returns a pointer to its copy of the result slot which the caller may set blocked to
-  // prevent the slot from being called. The pointer must be NULL if the result slot was already called
-  // because the resolve was synchronous.
-  const slot_resolver_type& resolver() const                  { return m_slotResolver; }
-  void                set_resolver(const slot_resolver_type& s) { m_slotResolver = s; }
+  // The resolver returns a pointer to its copy of the result slot
+  // which the caller may set blocked to prevent the slot from being
+  // called. The pointer must be NULL if the result slot was already
+  // called because the resolve was synchronous.
+  const slot_resolver_type& resolver() const                             { return m_slotResolver; }
+  void                      set_resolver(const slot_resolver_type& s)    { m_slotResolver = s; }
 
   // Since trackers need our port number, it doesn't get cleared after
   // 'listen_close()'. The client may change the reported port number,
@@ -168,6 +167,12 @@ public:
   port_type           listen_port() const                     { return m_listenPort; }
   void                set_listen_port(port_type p)            { m_listenPort = p; }
 
+  // The slot returns a ThrottlePair to use for the given address, or NULLs to use the default throttle.
+  const slot_address_throttle_type&
+                      address_throttle() const                { return m_slotAddressThrottle; }
+  void                set_address_throttle(const slot_address_throttle_type& s)
+                                                              { m_slotAddressThrottle = s; }
+
   // For internal usage.
   Listen*             listen()                                { return m_listen; }
 
@@ -193,6 +198,7 @@ private:
   slot_filter_type      m_slotFilter;
   signal_handshake_type m_signalHandshakeLog;
   slot_resolver_type    m_slotResolver;
+  slot_address_throttle_type  m_slotAddressThrottle;
 };
 
 }
