@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,7 +45,8 @@ namespace torrent {
 
 class TransferList : public std::vector<BlockList*> {
 public:
-  typedef std::vector<BlockList*> base_type;
+  typedef std::vector<BlockList*>                    base_type;
+  typedef std::vector<std::pair<int64_t, uint32_t> > completed_list_type;
 
   using base_type::value_type;
   using base_type::reference;
@@ -68,7 +69,14 @@ public:
   iterator            find(uint32_t index);
   const_iterator      find(uint32_t index) const;
 
+  const completed_list_type& completed_list() const { return m_completedList; }
+
+  uint32_t            succeeded_count() const { return m_succeededCount; }
+  uint32_t            failed_count() const { return m_failedCount; }
+
+  //
   // Internal to libTorrent:
+  //
 
   void                clear();
 
@@ -77,7 +85,7 @@ public:
 
   void                finished(BlockTransfer* transfer);
 
-  void                hash_succeded(uint32_t index);
+  void                hash_succeeded(uint32_t index, Chunk* chunk);
   void                hash_failed(uint32_t index, Chunk* chunk);
 
   typedef std::mem_fun1_t<void, ChunkSelector, uint32_t> slot_canceled_op;
@@ -103,7 +111,7 @@ private:
   void operator = (const TransferList&);
 
   unsigned int        update_failed(BlockList* blockList, Chunk* chunk);
-  void                mark_failed_peers(BlockList* blockList);
+  void                mark_failed_peers(BlockList* blockList, Chunk* chunk);
 
   void                retry_most_popular(BlockList* blockList, Chunk* chunk);
 
@@ -111,6 +119,11 @@ private:
   slot_completed_type m_slotCompleted;
   slot_queued_type    m_slotQueued;
   slot_corrupt_type   m_slotCorrupt;
+
+  completed_list_type m_completedList;
+
+  uint32_t            m_succeededCount;
+  uint32_t            m_failedCount;
 };
 
 }
