@@ -38,45 +38,12 @@
 
 #include <iostream>
 
+#include "rak/functional.h"
 #include "torrent/exceptions.h"
 #include "http.h"
 
 namespace torrent {
 
-template <typename Slot>
-inline void
-slot_list_call(const std::list<Slot>& slot_list) {
-  if (slot_list.empty())
-    return;
-
-  typename std::list<Slot>::const_iterator first = slot_list.begin();
-  typename std::list<Slot>::const_iterator next = slot_list.begin();
-
-  while (++next != slot_list.end()) {
-    (*first)();
-    first = next;
-  }
-
-  (*first)();
-}
-
-template <typename Slot, typename Arg1>
-inline void
-slot_list_call(const std::list<Slot>& slot_list, Arg1 arg1) {
-  if (slot_list.empty())
-    return;
-
-  typename std::list<Slot>::const_iterator first = slot_list.begin();
-  typename std::list<Slot>::const_iterator next = slot_list.begin();
-
-  while (++next != slot_list.end()) {
-    (*first)(arg1);
-    first = next;
-  }
-
-  (*first)(arg1);
-}
-
 Http::slot_http Http::m_factory;
 
 Http::~Http() {
@@ -84,7 +51,7 @@ Http::~Http() {
 
 void
 Http::trigger_done() {
-  torrent::slot_list_call(signal_done());
+  rak::slot_list_call(signal_done());
 
   if (m_flags & flag_delete_stream) {
     delete m_stream;
@@ -97,7 +64,7 @@ Http::trigger_done() {
 
 void
 Http::trigger_failed(const std::string& message) {
-  torrent::slot_list_call(signal_failed(), message);
+  rak::slot_list_call(signal_failed(), message);
 
   if (m_flags & flag_delete_stream) {
     delete m_stream;
