@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -47,12 +47,19 @@ public:
   friend class ConnectionList;
   friend class Handshake;
   friend class HandshakeManager;
+  friend class InitialSeeding;
   friend class PeerList;
   friend class ProtocolExtension;
 
   static const int flag_connected = (1 << 0);
   static const int flag_incoming  = (1 << 1);
   static const int flag_handshake = (1 << 2);
+  static const int flag_blocked   = (1 << 3);   // For initial seeding.
+  static const int flag_restart   = (1 << 4);
+  static const int flag_unwanted  = (1 << 5);
+  static const int flag_preferred = (1 << 6);
+
+  static const int mask_ip_table = flag_unwanted | flag_preferred;
 
   PeerInfo(const sockaddr* address);
   ~PeerInfo();
@@ -60,6 +67,10 @@ public:
   bool                is_connected() const                  { return m_flags & flag_connected; }
   bool                is_incoming() const                   { return m_flags & flag_incoming; }
   bool                is_handshake() const                  { return m_flags & flag_handshake; }
+  bool                is_blocked() const                    { return m_flags & flag_blocked; }
+  bool                is_restart() const                    { return m_flags & flag_restart; }
+  bool                is_unwanted() const                   { return m_flags & flag_unwanted; }
+  bool                is_preferred() const                  { return m_flags & flag_preferred; }
 
   int                 flags() const                         { return m_flags; }
 
