@@ -40,17 +40,25 @@
 #include <cstring>
 #include <cstdio>
 
+#include <stdexcept>
 #include <unistd.h>
 #include <torrent/exceptions.h>
 #include <torrent/event.h>
 
+#include "torrent.h"
 #include "poll_epoll.h"
-#include "thread_base.h"
+#include "utils/log.h"
+#include "utils/thread_base.h"
+#include "rak/error_number.h"
+#include "rak/timer.h"
 
 #ifdef USE_EPOLL
 #include <sys/epoll.h>
 #endif
 
+#define LT_LOG_EVENT(event, log_level, log_fmt, ...)                    \
+  lt_log_print(LOG_SOCKET_##log_level, "epoll->%s(%i): " log_fmt, event->type_name(), event->file_descriptor(), __VA_ARGS__);
+
 namespace torrent {
 
 #ifdef USE_EPOLL
@@ -71,6 +79,8 @@ PollEPoll::modify(Event* event, int op, uint32_t mask) {
   if (event_mask(event) == mask)
     return;
 
+  LT_LOG_EVENT(event, DEBUG, "Modify event: op:%hx mask:%hx.", op, mask);
+
   epoll_event e;
   e.data.u64 = 0; // Make valgrind happy? Remove please.
   e.data.fd = event->file_descriptor();
@@ -152,8 +162,8 @@ PollEPoll::perform() {
     if (itr->data.fd < 0 || (size_t)itr->data.fd >= m_table.size())
       continue;
 
-    if ((flags() & flag_waive_global_lock) && ThreadBase::global_queue_size() != 0)
-      ThreadBase::waive_global_lock();
+    if ((flags() & flag_waive_global_lock) && thread_base::global_queue_size() != 0)
+      thread_base::waive_global_lock();
 
     Table::iterator evItr = m_table.begin() + itr->data.fd;
 
@@ -176,6 +186,30 @@ PollEPoll::perform() {
   m_waitingEvents = 0;
 }
 
+void
+PollEPoll::do_poll(int64_t timeout_usec, int flags) {
+  rak::timer timeout = rak::timer(timeout_usec);
+
+  timeout += 10;
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::release_global_lock();
+    thread_base::entering_main_polling();
+  }
+
+  int status = poll((timeout.usec() + 999) / 1000);
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::leaving_main_polling();
+    thread_base::acquire_global_lock();
+  }
+
+  if (status == -1 && rak::error_number::current().value() != rak::error_number::e_intr)
+    throw std::runtime_error("Poll::work(): " + std::string(rak::error_number::current().c_str()));
+
+  perform();
+}
+
 uint32_t
 PollEPoll::open_max() const {
   return m_table.size();
@@ -183,12 +217,16 @@ PollEPoll::open_max() const {
 
 void
 PollEPoll::open(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Open event.", 0);
+
   if (event_mask(event) != 0)
     throw internal_error("PollEPoll::open(...) called but the file descriptor is active");
 }
 
 void
 PollEPoll::close(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Close event.", 0);
+
   if (event_mask(event) != 0)
     throw internal_error("PollEPoll::close(...) called but the file descriptor is active");
 
@@ -203,6 +241,8 @@ PollEPoll::close(Event* event) {
 
 void
 PollEPoll::closed(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Closed event.", 0);
+
   // Kernel removes closed FDs automatically, so just clear the mask and remove it from pending calls.
   // Don't touch if the FD was re-used before we received the close notification.
   if (m_table[event->file_descriptor()].second == event)
@@ -233,6 +273,8 @@ PollEPoll::in_error(Event* event) {
 
 void
 PollEPoll::insert_read(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert read.", 0);
+
   modify(event,
 	 event_mask(event) ? EPOLL_CTL_MOD : EPOLL_CTL_ADD,
 	 event_mask(event) | EPOLLIN);
@@ -240,6 +282,8 @@ PollEPoll::insert_read(Event* event) {
 
 void
 PollEPoll::insert_write(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert write.", 0);
+
   modify(event,
 	 event_mask(event) ? EPOLL_CTL_MOD : EPOLL_CTL_ADD,
 	 event_mask(event) | EPOLLOUT);
@@ -247,6 +291,8 @@ PollEPoll::insert_write(Event* event) {
 
 void
 PollEPoll::insert_error(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert error.", 0);
+
   modify(event,
 	 event_mask(event) ? EPOLL_CTL_MOD : EPOLL_CTL_ADD,
 	 event_mask(event) | EPOLLERR);
@@ -254,22 +300,25 @@ PollEPoll::insert_error(Event* event) {
 
 void
 PollEPoll::remove_read(Event* event) {
-  uint32_t mask = event_mask(event) & ~EPOLLIN;
+  LT_LOG_EVENT(event, DEBUG, "Remove read.", 0);
 
+  uint32_t mask = event_mask(event) & ~EPOLLIN;
   modify(event, mask ? EPOLL_CTL_MOD : EPOLL_CTL_DEL, mask);
 }
 
 void
 PollEPoll::remove_write(Event* event) {
-  uint32_t mask = event_mask(event) & ~EPOLLOUT;
+  LT_LOG_EVENT(event, DEBUG, "Remove write.", 0);
 
+  uint32_t mask = event_mask(event) & ~EPOLLOUT;
   modify(event, mask ? EPOLL_CTL_MOD : EPOLL_CTL_DEL, mask);
 }
 
 void
 PollEPoll::remove_error(Event* event) {
-  uint32_t mask = event_mask(event) & ~EPOLLERR;
+  LT_LOG_EVENT(event, DEBUG, "Remove error.", 0);
 
+  uint32_t mask = event_mask(event) & ~EPOLLERR;
   modify(event, mask ? EPOLL_CTL_MOD : EPOLL_CTL_DEL, mask);
 }
 
@@ -280,6 +329,7 @@ PollEPoll::~PollEPoll() {}
 
 int PollEPoll::poll(int msec) { throw internal_error("An PollEPoll function was called, but it is disabled."); }
 void PollEPoll::perform() { throw internal_error("An PollEPoll function was called, but it is disabled."); }
+void PollEPoll::do_poll(int64_t timeout_usec, int flags) { throw internal_error("An PollEPoll function was called, but it is disabled."); }
 uint32_t PollEPoll::open_max() const { throw internal_error("An PollEPoll function was called, but it is disabled."); }
 
 void PollEPoll::open(torrent::Event* event) {}
