@@ -40,13 +40,17 @@
 
 #include <algorithm>
 #include <unistd.h>
+#include <stdexcept>
 #include <rak/error_number.h>
-#include <rak/functional.h>
 #include <torrent/exceptions.h>
 #include <torrent/event.h>
 
 #include "poll_kqueue.h"
-#include "thread_base.h"
+#include "torrent.h"
+#include "rak/timer.h"
+#include "rak/error_number.h"
+#include "utils/log.h"
+#include "utils/thread_base.h"
 
 #ifdef USE_KQUEUE
 #include <sys/types.h>
@@ -57,6 +61,9 @@
 
 #include <assert.h>
 
+#define LT_LOG_EVENT(event, log_level, log_fmt, ...)                    \
+  lt_log_print(LOG_SOCKET_##log_level, "kqueue->%s(%i): " log_fmt, event->type_name(), event->file_descriptor(), __VA_ARGS__);
+
 namespace torrent {
 
 #ifdef USE_KQUEUE
@@ -90,6 +97,8 @@ PollKQueue::flush_events() {
 
 void
 PollKQueue::modify(Event* event, unsigned short op, short mask) {
+  LT_LOG_EVENT(event, DEBUG, "Modify event: op:%hx mask:%hx changed:%u.", op, mask, m_changedEvents);
+
   // Flush the changed filters to the kernel if the buffer is full.
   if (m_changedEvents == m_maxEvents) {
     if (kevent(m_fd, m_changes, m_changedEvents, NULL, 0, NULL) == -1)
@@ -213,8 +222,8 @@ PollKQueue::perform() {
     if (itr->ident >= m_table.size())
       continue;
 
-    if ((flags() & flag_waive_global_lock) && ThreadBase::global_queue_size() != 0)
-      ThreadBase::waive_global_lock();
+    if ((flags() & flag_waive_global_lock) && thread_base::global_queue_size() != 0)
+      thread_base::waive_global_lock();
 
     Table::iterator evItr = m_table.begin() + itr->ident;
 
@@ -236,6 +245,29 @@ PollKQueue::perform() {
   m_waitingEvents = 0;
 }
 
+void
+PollKQueue::do_poll(int64_t timeout_usec, int flags) {
+  rak::timer timeout = rak::timer(timeout_usec);
+  timeout += 10;
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::release_global_lock();
+    thread_base::entering_main_polling();
+  }
+
+  int status = poll((timeout.usec() + 999) / 1000);
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::leaving_main_polling();
+    thread_base::acquire_global_lock();
+  }
+
+  if (status == -1 && rak::error_number::current().value() != rak::error_number::e_intr)
+    throw std::runtime_error("Poll::work(): " + std::string(rak::error_number::current().c_str()));
+
+  perform();
+}
+
 uint32_t
 PollKQueue::open_max() const {
   return m_table.size();
@@ -243,12 +275,16 @@ PollKQueue::open_max() const {
 
 void
 PollKQueue::open(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Open event.", 0);
+
   if (event_mask(event) != 0)
     throw internal_error("PollKQueue::open(...) called but the file descriptor is active");
 }
 
 void
 PollKQueue::close(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Close event.", 0);
+
 #if KQUEUE_SOCKET_ONLY
   if (event->file_descriptor() == 0) {
     m_stdinEvent = NULL;
@@ -273,6 +309,8 @@ PollKQueue::close(Event* event) {
 
 void
 PollKQueue::closed(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Closed event.", 0);
+
 #if KQUEUE_SOCKET_ONLY
   if (event->file_descriptor() == 0) {
     m_stdinEvent = NULL;
@@ -318,6 +356,7 @@ PollKQueue::insert_read(Event* event) {
   if (event_mask(event) & flag_read)
     return;
 
+  LT_LOG_EVENT(event, DEBUG, "Insert read.", 0);
   set_event_mask(event, event_mask(event) | flag_read);
 
 #if KQUEUE_SOCKET_ONLY
@@ -335,12 +374,14 @@ PollKQueue::insert_write(Event* event) {
   if (event_mask(event) & flag_write)
     return;
 
+  LT_LOG_EVENT(event, DEBUG, "Insert write.", 0);
   set_event_mask(event, event_mask(event) | flag_write);
   modify(event, EV_ADD, EVFILT_WRITE);
 }
 
 void
 PollKQueue::insert_error(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert error.", 0);
 }
 
 void
@@ -348,6 +389,7 @@ PollKQueue::remove_read(Event* event) {
   if (!(event_mask(event) & flag_read))
     return;
 
+  LT_LOG_EVENT(event, DEBUG, "Remove read.", 0);
   set_event_mask(event, event_mask(event) & ~flag_read);
 
 #if KQUEUE_SOCKET_ONLY
@@ -365,12 +407,14 @@ PollKQueue::remove_write(Event* event) {
   if (!(event_mask(event) & flag_write))
     return;
 
+  LT_LOG_EVENT(event, DEBUG, "Remove write.", 0);
   set_event_mask(event, event_mask(event) & ~flag_write);
   modify(event, EV_DELETE, EVFILT_WRITE);
 }
 
 void
 PollKQueue::remove_error(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Remove error.", 0);
 }
 
 #else // USE_QUEUE
@@ -393,6 +437,11 @@ PollKQueue::perform() {
   throw internal_error("An PollKQueue function was called, but it is disabled.");
 }
 
+void
+PollKQueue::do_poll(int64_t timeout_usec, int flags) {
+  throw internal_error("An PollKQueue function was called, but it is disabled.");
+}
+
 uint32_t
 PollKQueue::open_max() const {
   throw internal_error("An PollKQueue function was called, but it is disabled.");
