@@ -38,6 +38,7 @@
 
 #include <algorithm>
 
+#include <stdexcept>
 #include <unistd.h>
 #include <sys/time.h>
 
@@ -47,10 +48,19 @@
 #include "event.h"
 #include "exceptions.h"
 #include "poll_select.h"
-#include "thread_base.h"
+#include "torrent.h"
+#include "rak/timer.h"
+#include "rak/error_number.h"
+#include "utils/log.h"
+#include "utils/thread_base.h"
+
+#define LT_LOG_EVENT(event, log_level, log_fmt, ...)                    \
+  lt_log_print(LOG_SOCKET_##log_level, "select->%s(%i): " log_fmt, event->type_name(), event->file_descriptor(), __VA_ARGS__);
 
 namespace torrent {
 
+Poll::slot_poll Poll::m_slot_create_poll;
+
 template <typename _Operation>
 struct poll_check_t {
   poll_check_t(Poll* p, fd_set* s, _Operation op) : m_poll(p), m_set(s), m_op(op) {}
@@ -71,8 +81,8 @@ struct poll_check_t {
       // We waive the global lock after an event has been processed in
       // order to ensure that 's' doesn't get removed before the op is
       // called.
-      if ((m_poll->flags() & Poll::flag_waive_global_lock) && ThreadBase::global_queue_size() != 0)
-        ThreadBase::waive_global_lock();
+      if ((m_poll->flags() & Poll::flag_waive_global_lock) && thread_base::global_queue_size() != 0)
+        thread_base::waive_global_lock();
     }
   }
 
@@ -203,6 +213,45 @@ PollSelect::perform(fd_set* readSet, fd_set* writeSet, fd_set* exceptSet) {
 		poll_check(this, writeSet, std::mem_fun(&Event::event_write)));
 }
 
+void
+PollSelect::do_poll(int64_t timeout_usec, int flags) {
+  rak::timer timeout = rak::timer(timeout_usec);
+
+  timeout += 10;
+
+  uint32_t set_size = open_max();
+
+  char read_set_buffer[set_size];
+  char write_set_buffer[set_size];
+  char error_set_buffer[set_size];
+  fd_set* read_set = (fd_set*)read_set_buffer;
+  fd_set* write_set = (fd_set*)write_set_buffer;
+  fd_set* error_set = (fd_set*)error_set_buffer;
+  std::memset(read_set_buffer, 0, set_size);
+  std::memset(write_set_buffer, 0, set_size);
+  std::memset(error_set_buffer, 0, set_size);
+
+  unsigned int maxFd = fdset(read_set, write_set, error_set);
+  timeval t = timeout.tval();
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::entering_main_polling();
+    thread_base::release_global_lock();
+  }
+
+  int status = select(maxFd + 1, read_set, write_set, error_set, &t);
+
+  if (!(flags & poll_worker_thread)) {
+    thread_base::leaving_main_polling();
+    thread_base::acquire_global_lock();
+  }
+
+  if (status == -1 && rak::error_number::current().value() != rak::error_number::e_intr)
+    throw std::runtime_error("Poll::work(): " + std::string(rak::error_number::current().c_str()));
+
+  perform(read_set, write_set, error_set);
+}
+
 inline static void
 log_poll_open(Event* event) {
 #ifdef LT_LOG_POLL_OPEN
@@ -224,6 +273,8 @@ log_poll_open(Event* event) {
 
 void
 PollSelect::open(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Open event.", 0);
+
   if ((uint32_t)event->file_descriptor() >= m_readSet->max_size())
     throw internal_error("Tried to add a socket to PollSelect that is larger than PollSelect::get_open_max()");
 
@@ -233,6 +284,8 @@ PollSelect::open(Event* event) {
 
 void
 PollSelect::close(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Close event.", 0);
+
   if ((uint32_t)event->file_descriptor() >= m_readSet->max_size())
     throw internal_error("PollSelect::close(...) called with an invalid file descriptor");
 
@@ -242,6 +295,8 @@ PollSelect::close(Event* event) {
 
 void
 PollSelect::closed(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Closed event.", 0);
+
   // event->get_fd() was closed, remove it from the sets.
   m_readSet->erase(event);
   m_writeSet->erase(event);
@@ -265,31 +320,37 @@ PollSelect::in_error(Event* event) {
 
 void
 PollSelect::insert_read(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert read.", 0);
   m_readSet->insert(event);
 }
 
 void
 PollSelect::insert_write(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert write.", 0);
   m_writeSet->insert(event);
 }
 
 void
 PollSelect::insert_error(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Insert error.", 0);
   m_exceptSet->insert(event);
 }
 
 void
 PollSelect::remove_read(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Remove read.", 0);
   m_readSet->erase(event);
 }
 
 void
 PollSelect::remove_write(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Remove write.", 0);
   m_writeSet->erase(event);
 }
 
 void
 PollSelect::remove_error(Event* event) {
+  LT_LOG_EVENT(event, DEBUG, "Remove error.", 0);
   m_exceptSet->erase(event);
 }
 
