@@ -391,24 +391,43 @@ tracker_next_timeout_promiscuous(Tracker* tracker) {
       !tracker->is_usable())
     return ~uint32_t();
 
-  std::pair<int, int> timeout_base;
+  int32_t interval;
 
   if (tracker->failed_counter())
-    timeout_base = std::make_pair(5, tracker->failed_counter() - 1);
-  else if (tracker->latest_sum_peers() < 10)
-    timeout_base = std::make_pair(10, tracker->success_counter());
-  else if (tracker->latest_new_peers() < 10)
-    timeout_base = std::make_pair(30, tracker->success_counter());
+    interval = 5 << std::min<int>(tracker->failed_counter() - 1, 6);
+  else if (tracker->success_counter() < 2 && tracker->latest_sum_peers() < 10)
+    interval = 10 << tracker->success_counter();
   else
-    // We got peers from this tracker, re-request a bit sooner.
-    timeout_base = std::make_pair(5, tracker->success_counter());
+    interval = tracker->normal_interval();
 
-  int32_t min_interval = std::min((int)tracker->min_interval(), 600);
+  int32_t min_interval = std::min(tracker->min_interval(), (uint32_t)600);
+  int32_t use_interval = std::min(interval, min_interval);
 
-  int32_t tracker_timeout = std::min(min_interval, (timeout_base.first << std::min(timeout_base.second, 6)));
   int32_t since_last = cachedTime.seconds() - (int32_t)tracker->activity_time_last();
 
-  return std::max(tracker_timeout - since_last, 0);
+  return std::max(use_interval - since_last, 0);
+}
+
+TrackerList::iterator
+tracker_find_preferred(TrackerList::iterator first, TrackerList::iterator last, uint32_t* next_timeout) {
+  TrackerList::iterator preferred = last;
+  uint32_t preferred_time_last = ~uint32_t();
+
+  for (; first != last; first++) {
+    uint32_t tracker_timeout = tracker_next_timeout_promiscuous(*first);
+
+    if (tracker_timeout != 0) {
+      *next_timeout = std::min(tracker_timeout, *next_timeout);
+      continue;
+    }
+
+    if ((*first)->activity_time_last() < preferred_time_last) {
+      preferred = first;
+      preferred_time_last = (*first)->activity_time_last();
+    }
+  }
+
+  return preferred;
 }
 
 void
@@ -416,18 +435,44 @@ TrackerController::do_timeout() {
   if (!(m_flags & flag_active) || !m_tracker_list->has_usable())
     return;
 
+  priority_queue_erase(&taskScheduler, &m_private->task_timeout);
+
   int send_state = current_send_state();
 
   if ((m_flags & (flag_promiscuous_mode | flag_requesting))) {
     uint32_t next_timeout = ~uint32_t();
 
-    for (TrackerList::iterator itr = m_tracker_list->begin(); itr != m_tracker_list->end(); itr++) {
-      uint32_t tracker_timeout = tracker_next_timeout_promiscuous(*itr);
+    TrackerList::iterator itr = m_tracker_list->begin();
 
-      if (tracker_timeout == 0)
-        m_tracker_list->send_state_itr(itr, send_state);
-      else
-        next_timeout = std::min(tracker_timeout, next_timeout);
+    while (itr != m_tracker_list->end()) {
+      uint32_t group = (*itr)->group();
+
+      if (m_tracker_list->has_active_not_scrape_in_group(group)) {
+        itr = m_tracker_list->end_group(group);
+        continue;
+      }
+
+      TrackerList::iterator group_end = m_tracker_list->end_group((*itr)->group());
+      TrackerList::iterator preferred = itr;
+
+      if (!(*itr)->is_usable() || (*itr)->failed_counter()) {
+        // The selected tracker in the group is either disabled or not
+        // reachable, try the others to find a new one to use.
+        preferred = tracker_find_preferred(preferred, group_end, &next_timeout);
+
+      } else {
+        uint32_t tracker_timeout = tracker_next_timeout_promiscuous(*preferred);
+
+        if (tracker_timeout != 0) {
+          next_timeout = std::min(tracker_timeout, next_timeout);
+          preferred = group_end;
+        }
+      }
+
+      if (preferred != group_end)
+        m_tracker_list->send_state_itr(preferred, send_state);
+
+      itr = group_end;
     }
 
     if (next_timeout != ~uint32_t())
@@ -447,7 +492,7 @@ TrackerController::do_timeout() {
     } else {
       TrackerList::iterator itr = m_tracker_list->find_next_to_request(m_tracker_list->begin());
 
-      int32_t next_timeout = (*itr)->failed_counter() == 0 ? (*itr)->success_time_next() : (*itr)->failed_time_next();
+      int32_t next_timeout = (*itr)->activity_time_next();
 
       if (next_timeout <= cachedTime.seconds())
         m_tracker_list->send_state_itr(itr, send_state);
@@ -462,8 +507,29 @@ TrackerController::do_timeout() {
 
 void
 TrackerController::do_scrape() {
-  std::for_each(m_tracker_list->begin(), m_tracker_list->end(),
-                tr1::bind(&TrackerList::send_scrape, m_tracker_list, tr1::placeholders::_1));
+  TrackerList::iterator itr = m_tracker_list->begin();
+
+  while (itr != m_tracker_list->end()) {
+    uint32_t group = (*itr)->group();
+
+    if (m_tracker_list->has_active_in_group(group)) {
+      itr = m_tracker_list->end_group(group);
+      continue;
+    }
+
+    TrackerList::iterator group_end = m_tracker_list->end_group((*itr)->group());
+
+    while (itr != group_end) {
+      if ((*itr)->can_scrape() && (*itr)->is_usable()) {
+        m_tracker_list->send_scrape(*itr);
+        break;
+      }
+
+      itr++;
+    }
+
+    itr = group_end;
+  }  
 }
 
 uint32_t
