@@ -21,6 +21,8 @@ class tracker_list_test : public CppUnit::TestFixture {
 
   CPPUNIT_TEST(test_scrape_success);
   CPPUNIT_TEST(test_scrape_failure);
+
+  CPPUNIT_TEST(test_has_active);
   CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -43,6 +45,8 @@ public:
 
   void test_scrape_success();
   void test_scrape_failure();
+
+  void test_has_active();
 };
 
 class TrackerTest : public torrent::Tracker {
@@ -92,6 +96,8 @@ private:
 extern uint32_t return_new_peers;
 inline uint32_t increment_value(int* value) { (*value)++; return return_new_peers; }
 
+bool check_has_active_in_group(const torrent::TrackerList* tracker_list, const char* states, bool scrape);
+
 #define TRACKER_SETUP()                                                 \
   torrent::TrackerList tracker_list;                                    \
   int success_counter = 0;                                              \
@@ -106,3 +112,45 @@ inline uint32_t increment_value(int* value) { (*value)++; return return_new_peer
 #define TRACKER_INSERT(group, name)                             \
   TrackerTest* name = new TrackerTest(&tracker_list, "");       \
   tracker_list.insert(group, name);
+
+#define TEST_TRACKER_IS_BUSY(tracker, state)            \
+  CPPUNIT_ASSERT(state == '0' ||  tracker->is_busy());  \
+  CPPUNIT_ASSERT(state == '1' || !tracker->is_busy());
+
+#define TEST_MULTI3_IS_BUSY(original, rearranged)                 \
+  TEST_TRACKER_IS_BUSY(tracker_0_0, original[0]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_0_1, original[1]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_1_0, original[2]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_2_0, original[3]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_3_0, original[4]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_list[0], rearranged[0]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[1], rearranged[1]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[2], rearranged[2]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[3], rearranged[3]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[4], rearranged[4]);
+
+#define TEST_GROUP_IS_BUSY(original, rearranged)                  \
+  TEST_TRACKER_IS_BUSY(tracker_0_0, original[0]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_0_1, original[1]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_0_2, original[2]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_1_0, original[3]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_1_1, original[4]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_2_0, original[5]);                 \
+  TEST_TRACKER_IS_BUSY(tracker_list[0], rearranged[0]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[1], rearranged[1]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[2], rearranged[2]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[3], rearranged[3]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[4], rearranged[4]);           \
+  TEST_TRACKER_IS_BUSY(tracker_list[5], rearranged[5]);
+
+#define TEST_TRACKERS_IS_BUSY_5(original, rearranged)   \
+  TEST_TRACKER_IS_BUSY(tracker_0, original[0]);         \
+  TEST_TRACKER_IS_BUSY(tracker_1, original[1]);         \
+  TEST_TRACKER_IS_BUSY(tracker_2, original[2]);         \
+  TEST_TRACKER_IS_BUSY(tracker_3, original[3]);         \
+  TEST_TRACKER_IS_BUSY(tracker_4, original[4]);         \
+  TEST_TRACKER_IS_BUSY(tracker_list[0], rearranged[0]); \
+  TEST_TRACKER_IS_BUSY(tracker_list[1], rearranged[1]); \
+  TEST_TRACKER_IS_BUSY(tracker_list[2], rearranged[2]); \
+  TEST_TRACKER_IS_BUSY(tracker_list[3], rearranged[3]); \
+  TEST_TRACKER_IS_BUSY(tracker_list[4], rearranged[4]);
